indexing
	description: "Mealy automata"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	AUTO
create
	make

feature --Initialization
	make (prototype: STATE) is
			-- Make automaton with states that are clones of `prototype'
		require
			prototype_exists: prototype /= Void
		local
			i: INTEGER
		do
			create states.make (1, prototype.state_count)
			from
				i := states.lower
			until
				i > states.upper
			loop
				states[i] := prototype.deep_twin
				i := i + 1
			end
			restart
		end


feature -- Access
	actions: BIT_VECTOR
		-- Latest transition's output

	state_count: INTEGER is
			-- Number of states
		do
			Result := states.count
		end

	predicate_count: INTEGER is
			-- Number of predicates in input to `Current'
		do
			Result := states[states.lower].predicate_count
		end

	action_count: INTEGER is
			-- Number of actions in output of `Current'
		do
			Result := states[states.lower].action_count
		end

feature -- Status setting
	restart is
			-- Return back to initial state
	do
		current_state := states[states.lower].Initial_state
	ensure
		initial_state_is_current: current_state = states[states.lower].Initial_state
	end

feature -- Transformation
	do_transition (in: BIT_VECTOR) is
			-- Changes current state and output according to `in'
		require
			correct_input_size: in /= Void and then in.count = predicate_count
		do
			states[current_state].set_input(in)
			actions := states[current_state].actions
			current_state := states[current_state].next_state
		end

feature --{NONE} -- Implementation
	states: ARRAY[STATE]
			-- An array of automaton's states
	current_state: INTEGER
			-- Index of aoutomaton's current state in the `states' array

invariant
	states_exists: states /= Void
	predicate_count_not_negative: predicate_count >= 0
	action_count_not_negative: action_count >= 0
	state_count_positive: state_count > 0
	at_least_one_state_exists: states[states.lower] /= Void
	current_state_not_too_small: current_state >= states.lower
	current_state_not_too_large: current_state <= states.upper
	current_state_exists: states[current_state] /= Void
	state_count_correct: states[current_state].state_count = state_count
	-- predicate_count_the_same: forall i in [1..state_count] states[i].predicate_count = predicate_count
	-- action_count_the_same: forall i in [1..state_count] states[i].action_count = action_count
end
