indexing
	description: "Evolving automated control objects"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	GENETIC_AUTOMATED_OBJECT

inherit
	AUTOMATED_OBJECT
	redefine
		controller, computer, make
	end

	INDIVIDUAL
	redefine
		instance, notify_of_new_generation
	end
create
	make

feature -- Initialization
	make (a: GENETIC_AUTO; co: GENETIC_CONTROL_OBJECT) is
			-- Call `Precursor', set `step_count' to default (defined in control object) and `fitness' to default
		do
			Precursor (a, co)
			set_step_count (co.default_step_count)
			set_fitness_to_default
		ensure then
			step_count_default: step_count = co.default_step_count
			fitness_default: fitness = default_fitness
		end

	randomize is
			-- Randomize controller
		do
			controller.randomize
		end

feature -- Access
	mutation_index: REAL is
			-- Controller's mutation_index
		do
			Result := controller.mutation_index
		end

	step_count: INTEGER
			-- Number of emulation steps used in evaluation

feature -- Duplication
	instance: like Current is
			-- Member of `POPULATION' to which `Current' is used as `prototype'
			-- `controller' is copied, `computer' is shared
		do
			create Result.make(controller.deep_twin, computer)
			Result.set_step_count(step_count)
		end

feature -- Status report
	is_compatible_with (other: like Current): BOOLEAN is
			-- Can `Current' and `other' have chilren?
			-- Yes, if their controllers are compatible and computers are the same
		do
			Result := (controller.is_compatible_with(other.controller)) and then
				(computer = other.computer) and then (step_count = other.step_count)
		ensure then
			compatible_definition: Result = (controller.is_compatible_with(other.controller)) and then
				(computer = other.computer) and then (step_count = other.step_count)
		end

feature -- Measurement
	evaluate is
			-- Calculate fitness
		local
			i: INTEGER
		do
			restart
			from
				i := 1
			until
--				i > step_count
				computer.timeout
			loop
				do_step
				i := i + 1
			end
			fitness := computer.fitness
		end

feature -- Basic operations
	mutant: like Current is
			-- Automated object wit a mutated version of controller and same computer
		do
			create Result.make(controller.mutant, computer)
			Result.set_step_count(step_count)
		ensure then
			same_computer: Result.computer = computer
			same_step_count: Result.step_count = step_count
		end


	children (other_parent: like Current): ARRAY [like Current] is
			-- A pair of children who's controllers are children of parents' controllers
			-- and computer is the same as both parents have
		local
			children_controllers: ARRAY[like controller]
			child: like Current
		do
			children_controllers := controller.children (other_parent.controller)

			create Result.make (1, 2)

			create child.make(children_controllers[1], computer)
			child.set_step_count (step_count)
			Result[1] := child

			create child.make(children_controllers[2], computer)
			child.set_step_count (step_count)
			Result[2] := child
		ensure then
			same_first_child_computer: Result[1].computer = computer
			same_second_child_computer: Result[2].computer = computer
			same_first_child_step_count: Result[1].step_count = step_count
			same_second_child_step_count: Result[2].step_count = step_count
		end

	set_mutation_index (p: REAL) is
			-- Make mutation_index equal to `p'
		do
			controller.set_mutation_index (p)
		end

	set_step_count (n: INTEGER) is
			-- Set `step_count' to `n'
		require
			n_positive: n > 0
		do
			step_count := n
		ensure
			step_count_set: step_count = n
		end

feature -- Miscellaneous
	notify_of_new_generation is
			-- Notify computer, it may change some of its parameters
		do
			computer.notify_of_new_generation
		end

feature {GENETIC_AUTOMATED_OBJECT} -- Implementation
	controller: GENETIC_AUTO
	computer: GENETIC_CONTROL_OBJECT

invariant
	step_count_positive: step_count > 0
end
