indexing
	description: "Individuals that are represented by their genetic code and are subject to natural selection (have fitness)"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	INDIVIDUAL

inherit
	GENETIC_CODE

feature -- Initialization
	set_fitness_to_default is
			-- Set `fitness' to `default_fitness'
		do
			fitness := default_fitness
		ensure
			fitness_default: fitness = default_fitness
		end

feature -- Access
	fitness: REAL
			-- Fitness of `Current'

	default_fitness : REAL is - 1.0
			-- Value of fitness before individual is evaluated		

feature -- Duplication
	instance: like Current is
			-- Member of `POPULATION' to which `Current' is used as `prototype'
			-- Default semantics is `deep_twin', may be redefined if population members have shared objects
		do
			Result := deep_twin
		end

feature -- Status report
	is_evaluated: BOOLEAN is
			-- Is `Current' already evaluated?
		do
			Result := fitness >= 0.0
		end

feature -- Measurement
	evaluate is
			-- Calculate fitness
		deferred
		ensure
			is_evaluated
		end

feature -- Miscellaneous
	notify_of_new_generation is
			-- To be called by anyone who evaluates individuals to notify first individual in generation before its evaluation
			-- Do nothing by default, may be redefined to perform some special action
			-- e.g. change environment to make individuals less specific
		do
		end

invariant
	evaluated_definition: is_evaluated = (fitness >= 0.0)
	fitness_constraint: is_evaluated xor (fitness = default_fitness)
	instance_compatible: is_compatible_with (instance)
end
