indexing
	description: "Pseudo-random sequnce with some additional features"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	MY_RANDOM

inherit
	RANDOM
	rename
		make as default_make
	end

	REAL_COMPARISON
create make

feature -- Initialization
	make is
			-- Initialize structure using time-dependent seed
		local
			t: TIME
			r: DOUBLE
		do
			create t.make_now
			r := t.fine_seconds
--			r := 0.5
			set_seed((default_seed*(r - r.floor_real_64)).floor)
			start
		end

feature -- Access
	bounded_item (min, max: INTEGER): INTEGER is
			-- Random integer from [min..max]
		require
			bounds_valid: max >= min
		do
			Result := (real_item*(max - min + 1)).floor + min
			if Result > max then Result := max
			end
		ensure
			result_not_too_small: Result >= min
			result_not_too_large: Result <= max
		end

	bounded_real_item (min, max: REAL): REAL is
			-- Random real from [min..max]
		require
			bounds_valid: max >= min
		do
			Result := min + (max - min)*real_item
		ensure
			result_not_too_small: Result >= min
			result_not_too_large: Result <= max
		end

	decide (p: REAL): BOOLEAN is
			-- True with probability `p', False otherwise
		require
			p_is_probability: (p >= 0.0) and then (p <= 1.0)
		do
			Result := real_item < p
		end

	sample (distribution: ARRAY [REAL]): INTEGER is
			-- Random index from [`distribution.lower', `distribution.upper'] with probability defined by `distribution'
	require
		distribution_exists: distribution /= Void
		distribution_valid: is_distribution_valid (distribution)
	local
		sum: REAL
	do
		from
			Result := distribution.lower
			sum := bounded_real_item (0.0, 1.0)
		until
			sum <= 0.0
		loop
			check Result <= distribution.upper end -- Because sum in [0, 1] and distribution.totak in [0, 1]
			sum := sum - distribution[Result]
			Result := Result + 1
		end

		if Result > distribution.lower then
			Result := Result - 1
		end
	ensure
		index_not_too_small: Result >= distribution.lower
		index_not_too_large: Result <= distribution.upper
	end

	is_distribution_valid (distribution: ARRAY [REAL]): BOOLEAN is
			-- Is `distribution' a valid discrete distribution?
		require
			distribution_exists: distribution /= Void
		local
			i: INTEGER
			has_negative: BOOLEAN
			sum: REAL
		do
			from
				i := distribution.lower
			until
				i > distribution.upper
			loop
				if distribution[i] < 0.0 then
					has_negative := True
					i := distribution.upper
				else
					sum := sum + distribution [i]
				end
				i := i + 1
			end
			Result := not has_negative and approx_equal (sum, 1.0)
		end
end
