indexing
	description: "Collections of individuals"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	POPULATION
inherit
	ARRAY[INDIVIDUAL]
	rename
		subarray as subpopulation
	end
create
	make

feature -- Initialization
	randomize (prototype: INDIVIDUAL) is
			-- Initialize each individual with `instance' of `prototype' and then randomize it
		require
			prototype_exist: prototype /= Void
		local
			i: INTEGER
			ind: INDIVIDUAL
		do
			from
				i := lower
			until
				i > upper
			loop
				ind := prototype.instance
				ind.randomize
				put (ind, i)
				i := i + 1
			end
		end

feature -- Basic operations
	best : INDIVIDUAL is
			-- Individual with greatest fitness from `Current'
		require
			at_least_one_exists: count >= 1
		local
			i: INTEGER
		do
			from
				Result := item(lower)
				i := lower + 1
			until
				i > upper
			loop
				if item(i).fitness > Result.fitness then
					Result := item(i)
				end
				i := i + 1
			end
		end

	selected(selected_count, candidate_competition: INTEGER) : POPULATION is
			-- `selected_count' best individuals, each chosen from `candidate_competition'
			-- random ones from `Current'
			-- (Natural selection, tournament method)
		require
			candidate_competition_not_too_small: candidate_competition >= 1
		local
			candidates: POPULATION
			i, j: INTEGER
		do
			create Result.make(1, selected_count)
			create candidates.make (1, candidate_competition)

			from
				i := Result.lower
			until
				i > Result.upper
			loop
				from
					j := 1
				until
					j > candidate_competition
				loop
					r.forth
					candidates[j] := item(r.bounded_item(lower, upper))
					j := j + 1
				end
				Result[i] := candidates.best
				i := i + 1
			end
		ensure
			result_count_is_selected_count: Result.count = selected_count
		end

	mutants: POPULATION is
			-- Individuals from `Current', possibly mutated
		local
			i: INTEGER
		do
			create Result.make(lower, upper)
			from
				i := Result.lower
			until
				i > Result.upper
			loop
				Result[i] := item(i).mutant
				i := i + 1
			end
		ensure
			lower_is_same: Result.lower = lower
			upper_is_same: Result.upper = upper
		end

	children(n, parents_competition: INTEGER): POPULATION is
			-- Population of size `n' filled with chilren of some individuals from `Current'
		require
			n_not_negative: n >= 0
			parents_competition_not_too_small: parents_competition >= 1
		local
			i: INTEGER
			first_parent, second_parent: INDIVIDUAL
			ch: ARRAY[INDIVIDUAL]
		do
			create Result.make(1, n)
			from
				i := Result.lower
			until
				i > Result.upper
			loop
				choose_parent(parents_competition)
				first_parent := chosen_parent
				choose_parent(parents_competition)
				second_parent := chosen_parent
				check first_parent.is_compatible_with(second_parent) end -- Because they are from (instance, mutant, children)-closure of `prototype'
				ch := first_parent.children(second_parent)
				Result[i] := ch[1]
				i := i + 1
				if i <= Result.upper then
					Result[i] := ch[2]
					i := i + 1
				end
			end
		ensure
			result_count_is_n: Result.count = n
		end

	evaluate is
			-- Evaluate population
		local
			i : INTEGER
		do
			from
				i := lower
				item(lower).notify_of_new_generation
			until
				i > upper
			loop
				item(i).evaluate
				i := i + 1
			end
		ensure
			-- all_evaluated: forall i: item(i).evaluated
		end


feature {NONE} -- Implementation
	chosen_parent: INDIVIDUAL
			-- Individual who was chosen to become a parent

	choose_parent (parents_competition: INTEGER) is
			-- Select one from `Current' to become `chosen_parent'
		require
			parents_competition_not_too_small: parents_competition >= 1
		do
			chosen_parent := selected(1, parents_competition)[1]
		end

	r: MY_RANDOM is
			-- Random sequence for implementation needs
		once
			create Result.make
		end

invariant
	r_exists: r /= Void
end
