indexing
	description: "States in which only some predicates are significant"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	REDUCED_TABLE_STATE

inherit
	STATE
		select
			predicate_count,
			current_input
		end

	FULL_TABLE_STATE
		rename
			predicate_count as significant_predicate_count,
			current_input as significant_input,
			make as make_full,
			make_compatible as make_compatible_full
		export {NONE}
			make_full, make_compatible_full
		redefine
			set_input,
			update_input_index
		end

create
	make, make_compatible

feature -- Initialization
	make (p, a, s, sp: INTEGER) is
			-- Create state with `p' predicates, of which `sp' are significant,
			-- `a' actions and capable of connecting to `s' other states
		require
			p_not_negative: p >= 0
			a_not_negative: a >= 0
			s_positive: s > 0
			sp_not_negative: sp >= 0
			sp_not_too_large: sp <= p
		local
			i: INTEGER
		do
			predicate_count := p
			create current_input.make (1, predicate_count)
			create significant_predicates.make (1, predicate_count)
			from
				i := significant_predicates.lower
			until
				i > sp
			loop
				significant_predicates [i] := True
				i := i + 1
			end
			make_full (sp, a, s)
		ensure
			predicate_count_set: predicate_count = p
			action_count_set: action_count = a
			state_count_set: state_count = s
			significant_predicate_count_set: significant_predicate_count = sp
		end

	make_compatible (co: CONTROL_OBJECT; s, sp: INTEGER) is
			-- Create a state with same `predicate_count' and `action_count' as `co' (only `sp' predicates are siginificant)
			-- and capable of connecting to `s' other states
		require
			co_exists: co /= Void
			sp_not_negative: sp >= 0
			sp_not_too_large: sp <= co.predicate_count
		do
			make(co.predicate_count, co.action_count, s, sp)
		ensure
			predicate_count_compatible: predicate_count = co.predicate_count
			action_count_compatible: action_count = co.action_count
			state_count_set: state_count = s
			significant_predicate_count_set: significant_predicate_count = sp
		end

feature -- Access
	predicate_count: INTEGER
			-- Total predicate count

	significant_predicates: BIT_VECTOR
			-- Predicates that are significant in this state

feature {AUTO} -- Transformation
	set_input (in: BIT_VECTOR) is
		-- Set `current_input' to `in'
		do
			current_input.copy (in)
			update_input_index
		end

feature {NONE} -- Implementation
	current_input: BIT_VECTOR
			-- Input to be used by `actions' and `next_state'

	update_input_index is
			-- Make `input_index' a correct representation of `current_input'
		local
			si, fi: INTEGER
		do
			create significant_input.make (1, significant_predicate_count)
			from
				si := significant_input.lower
				fi := current_input.lower
			until
				si > significant_predicate_count
			loop
				if significant_predicates [fi] then
					significant_input [si] := current_input [fi]
					si := si + 1
				end
				fi := fi + 1
			end
			input_index := significant_input.to_integer + 1
		end

invariant
	significant_predicate_count_not_too_large: significant_predicate_count <= predicate_count
	significant_predicate_count_right: significant_predicates.occurrences (True) = significant_predicate_count
end
