indexing
	description: "Trinary digits"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

expanded class
	TRINARY

inherit ANY
	redefine
		default_create, out
	end


create
	default_create, make_from_integer, make_from_boolean

convert
	make_from_integer ({INTEGER}),
	make_from_boolean ({BOOLEAN}),
	to_integer: {INTEGER},
	to_boolean: {BOOLEAN}

feature -- Initialization
	default_create is
			-- Create trinary with value `N'
		do
			item := N_integer
		end

	make_from_integer (i: INTEGER) is
			-- Create trinary with value `i'
		require
			valid_trinary: is_trinary (i)
		do
			item := i
		ensure
			item_set: item = i
		end

	make_from_boolean (b: BOOLEAN) is
			-- Create trinary with value `b'
		do
			if b then
				item := T_integer
			else
				item := F_integer
			end
		ensure
			t_if_true: b implies Current = T
			f_if_false: not b implies Current = F
		end

feature -- Access
	T: TRINARY is
			-- T constant
		once
			create Result.make_from_integer (T_integer)
		end

	F: TRINARY is
			-- F constant
		once
			create Result.make_from_integer (F_integer)
		end

	N: TRINARY is
			-- N constant
		once
			create Result.make_from_integer (N_integer)
		end

feature -- Conversion
	to_integer: INTEGER is
			-- Integer representation of trinary
		do
			Result := item
		end

	to_boolean: BOOLEAN is
			-- Boolean representation of trinary
		require
			true_or_false: Current = T or Current = F
		do
			if item = T_integer then
				Result := True
			end
		end

feature -- Status report
	is_trinary (i: INTEGER): BOOLEAN is
			-- Is `i' a valid integer representation of trinary
		do
			Result := i = F_integer or i = T_integer or i = N_integer
		end

feature -- Output
	out: STRING is
			-- String representation of trinary
		do
			inspect item
			when F_integer then Result := "F"
			when T_integer then Result := "T"
			when N_integer then Result := "NEVERMIND"
			end
		end


feature {NONE} -- Implementation
	F_integer: INTEGER is 0
	T_integer: INTEGER is 1
	N_integer: INTEGER is 2

	item: INTEGER

invariant
	item_is_trinary: is_trinary (item)
end
